import { getRendition, createDimension, getSourceSet } from 'ibm-wch-sdk-rendition-utils';
import { WchLoggerService, AbstractLifeCycleComponent, ZoneService } from 'ibm-wch-sdk-ng';
export { AbstractLifeCycleComponent as ɵa } from 'ibm-wch-sdk-ng';
import { Inject, Injectable, Injector, Directive, ElementRef, EventEmitter, Input, Output, Renderer2, ViewContainerRef, NgModule, Optional, SkipSelf } from '@angular/core';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { combineLatest } from 'rxjs/observable/combineLatest';
import { fromEvent } from 'rxjs/observable/fromEvent';
import { of } from 'rxjs/observable/of';
import { debounceTime } from 'rxjs/operators/debounceTime';
import { distinctUntilChanged } from 'rxjs/operators/distinctUntilChanged';
import { filter } from 'rxjs/operators/filter';
import { map } from 'rxjs/operators/map';
import { mapTo } from 'rxjs/operators/mapTo';
import { shareReplay } from 'rxjs/operators/shareReplay';
import { startWith } from 'rxjs/operators/startWith';
import { switchMap } from 'rxjs/operators/switchMap';
import { switchMapTo } from 'rxjs/operators/switchMapTo';
import { takeUntil } from 'rxjs/operators/takeUntil';
import { tap } from 'rxjs/operators/tap';
import { ReplaySubject } from 'rxjs/ReplaySubject';
import { CommonModule } from '@angular/common';
import { createConsumerOnSubject, createSetterOnSubject, identity } from 'ibm-wch-sdk-utils';
import { Observable } from 'rxjs/Observable';
import { empty } from 'rxjs/observable/empty';
import { concatMap } from 'rxjs/operators/concatMap';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const LOGGER = 'WchRenditionService';
class WchRenditionService {
    /**
     * @param {?} aInjector
     */
    constructor(aInjector) {
        const /** @type {?} */ that = this;
        const /** @type {?} */ loggerService = aInjector.get(WchLoggerService);
        const /** @type {?} */ logger = loggerService.get(LOGGER);
        /**
         * @param {?} aWidth
         * @param {?} aHeight
         * @param {?} aImage
         * @return {?}
         */
        function _getRenditionUrl(aWidth, aHeight, aImage) {
            // simply return the rendition URL
            return getRendition(aWidth, aHeight, aImage, logger).url;
        }
        /**
         * @param {?} aWidth
         * @param {?} aHeight
         * @param {?} aImage
         * @return {?}
         */
        function _getRendition(aWidth, aHeight, aImage) {
            return getRendition(aWidth, aHeight, aImage, logger);
        }
        // initialize the calls
        that.getRenditionUrl = _getRenditionUrl;
        that.getRendition = _getRendition;
    }
}
WchRenditionService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
WchRenditionService.ctorParameters = () => [
    { type: Injector, decorators: [{ type: Inject, args: [Injector,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/* Copyright IBM Corp. 2017 */
const KEY_ON_RENDERING_CONTEXT = 'onRenderingContext';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const _mathRound = Math.round;
const jsonStringify = JSON.stringify;
/**
 * Compares two dimension objects
 *
 * @param {?} aLeft   left object
 * @param {?} aRight  right object
 *
 * @return {?} true if the dimensions match, else false
 */
function _equalDimension(aLeft, aRight) {
    return (aLeft === aRight) || ((aLeft.w === aRight.w) && (aLeft.h === aRight.h));
}
/**
 * Compares two scaled image objects
 *
 * @param {?} aLeft   left object
 * @param {?} aRight  right object
 *
 * @return {?} true if the scaled images match, else false
 */
function _equalScaledImage(aLeft, aRight) {
    return (aLeft === aRight) || (_equalDimension(aLeft.img, aRight.img)
        && _equalDimension(aLeft.container, aRight.container) && (aLeft.url === aRight.url));
}
/**
 * Constructs the dimensions object based on an array
 *
 * @param {?} aArray  the array
 * @return {?} the dimensions object
 */
function _dimensionFromArray(aArray) {
    return createDimension(aArray[0], aArray[1]);
}
/**
 * Returns the measured dimension of an item
 *
 * @param {?} aElement  the element to measure
 * @param {?} aRenderer   the rendered used to access the parent
 * @param {?} aLogger
 * @return {?} the dimension
 */
function _getMeasuredDimension(aElement, aRenderer, aLogger) {
    // check
    let /** @type {?} */ ce = aElement;
    let /** @type {?} */ w = 0, /** @type {?} */ h = 0;
    while ((w <= 0) && ce) {
        ce = aRenderer.parentNode(ce);
        w = _mathRound(ce.clientWidth);
        h = _mathRound(ce.clientHeight);
    }
    // log this
    aLogger.info('Measured dimension', w, 'x', h, 'via', ce);
    // returns the measured dimension
    return createDimension(w, h);
}
const LOGGER$1 = 'WchRenditionDirective';
class RenditionDirective extends AbstractLifeCycleComponent {
    /**
     * @param {?} vcRef
     * @param {?} parentRef
     * @param {?} renderer
     * @param {?} aInjector
     */
    constructor(vcRef, parentRef, renderer, aInjector) {
        super();
        this.onImage = new EventEmitter();
        // quick check
        const /** @type {?} */ that = this;
        const /** @type {?} */ loggerService = aInjector.get(WchLoggerService);
        const /** @type {?} */ logger = loggerService.get(LOGGER$1);
        // log the construction
        logger.info('Constructing ...');
        // the basic hooks
        const /** @type {?} */ onDestroy = that.onOnDestroy.pipe(shareReplay(1));
        const /** @type {?} */ onInit = that.onOnInit.pipe(shareReplay(1));
        // dynamic elements
        const /** @type {?} */ onImage = new ReplaySubject(1);
        const /** @type {?} */ onRelWidth = new BehaviorSubject(0);
        const /** @type {?} */ onRelHeight = new BehaviorSubject(0);
        const /** @type {?} */ onImgWidth = new BehaviorSubject(0);
        const /** @type {?} */ onImgHeight = new BehaviorSubject(0);
        // declare the getters
        Object.defineProperties(that, {
            'wchRendition': that.describeSetter(onImage),
            'relWidth': that.describeSetter(onRelWidth),
            'relHeight': that.describeSetter(onRelHeight),
            'imgWidth': that.describeSetter(onImgWidth),
            'imgHeight': that.describeSetter(onImgHeight)
        });
        // exposes the current element
        const /** @type {?} */ onCurrentElement = onInit.pipe(mapTo(/** @type {?} */ (parentRef.nativeElement)), filter(Boolean), distinctUntilChanged());
        // listens for the configured dimension
        const /** @type {?} */ onConfiguredDimension = onInit.pipe(switchMapTo(combineLatest(onImgWidth, onImgHeight)), map(_dimensionFromArray), distinctUntilChanged(_equalDimension), tap(d => logger.info('config', d)));
        // relative dimensions
        const /** @type {?} */ onRelDimension = onInit.pipe(switchMapTo(combineLatest(onRelWidth, onRelHeight)), map(_dimensionFromArray), distinctUntilChanged(_equalDimension), tap(d => logger.info('rel', d)));
        // measured dimensions
        const /** @type {?} */ onMeasuredDimension = onCurrentElement.pipe(switchMap(ce => fromEvent(window, 'resize').pipe(startWith('start'), debounceTime(300), mapTo(_getMeasuredDimension(ce, renderer, logger)))), distinctUntilChanged(_equalDimension), tap(d => logger.info('measured', d)));
        // actual dimensions to use
        const /** @type {?} */ onSrcDimension = onConfiguredDimension.pipe(switchMap(d => {
            // shortcut
            const /** @type {?} */ dw = d.w, /** @type {?} */ dh = d.h;
            return ((dw > 0) && (dh > 0)) ? of(d) : onMeasuredDimension.pipe(map(dm => {
                // shortcut
                const /** @type {?} */ mw = dm.w, /** @type {?} */ mh = dm.h;
                return (dw > 0) ? createDimension(dw, mh) : ((dh > 0) ? createDimension(mw, dh) : dm);
            }));
        }), distinctUntilChanged(_equalDimension), tap(d => logger.info('src', d)));
        // compute the target dimensions
        const /** @type {?} */ onDstDimension = onSrcDimension.pipe(switchMap(sd => {
            // shortcut
            const /** @type {?} */ sw = sd.w, /** @type {?} */ sh = sd.h;
            return onRelDimension.pipe(map(rd => {
                // shortcut
                const /** @type {?} */ rw = rd.w, /** @type {?} */ rh = rd.h;
                return (rw > 0) ? ((rh > 0) ? createDimension(rw * sh, rh * sw) : createDimension(rw * sh, sh))
                    : ((rh > 0) ? createDimension(sw, rh * sw) : sd);
            }));
        }), distinctUntilChanged(_equalDimension), tap(d => logger.info('dst', d)));
        // the scaled image
        const /** @type {?} */ onScaledImage = combineLatest(onImage.pipe(filter(Boolean)), onDstDimension).pipe(map(([img, d]) => getRendition(d.w, d.h, img, logger)), distinctUntilChanged(_equalScaledImage), tap(d => logger.info('url', jsonStringify(d))));
        // availability of the rendering context, detected from the hosting component
        const /** @type {?} */ onRenderingContext = onInit.pipe(switchMap(() => {
            // TODO this is a hack, but is there a better alternative?
            const /** @type {?} */ cmp = (/** @type {?} */ (vcRef.injector)).view.component;
            // access the rendering context property
            return /** @type {?} */ (cmp[KEY_ON_RENDERING_CONTEXT]);
        }));
        // availability of the base URL
        const /** @type {?} */ onDeliveryUrl = onRenderingContext.pipe(filter(Boolean), map(rc => rc.context.hub.deliveryUrl.origin), distinctUntilChanged());
        // compute the full URL
        combineLatest(onDeliveryUrl, onScaledImage).pipe(map(([base, img]) => Object.assign({}, img, { url: base + img.url })), distinctUntilChanged(_equalScaledImage), tap(img => logger.info('result', img)), takeUntil(onDestroy))
            .subscribe(img => {
            const /** @type {?} */ el = parentRef.nativeElement;
            renderer.setAttribute(el, 'src', img.url);
            renderer.setAttribute(el, 'width', img.container.w.toString());
            renderer.setAttribute(el, 'height', img.container.h.toString());
            // notify our clients
            that.onImage.emit(img);
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        super.ngOnInit();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        super.ngOnDestroy();
    }
}
RenditionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[wchRendition]'
            },] },
];
/** @nocollapse */
RenditionDirective.ctorParameters = () => [
    { type: ViewContainerRef, },
    { type: ElementRef, },
    { type: Renderer2, },
    { type: Injector, decorators: [{ type: Inject, args: [Injector,] },] },
];
RenditionDirective.propDecorators = {
    "wchRendition": [{ type: Input },],
    "relHeight": [{ type: Input },],
    "relWidth": [{ type: Input },],
    "imgWidth": [{ type: Input },],
    "imgHeight": [{ type: Input },],
    "onImage": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SrcSetDirective extends AbstractLifeCycleComponent {
    /**
     * @param {?} vcRef
     * @param {?} parentRef
     * @param {?} renderer
     */
    constructor(vcRef, parentRef, renderer) {
        super();
        // quick check
        const /** @type {?} */ that = this;
        // the basic hooks
        const /** @type {?} */ onDestroy = that.onOnDestroy;
        const /** @type {?} */ onInit = that.onOnInit;
        // dynamic elements
        const /** @type {?} */ onImage = new ReplaySubject(1);
        // declare the getters
        Object.defineProperties(that, {
            'wchSrcSet': this.describeSetter(onImage)
        });
        // availability of the rendering context, detected from the hosting component
        const /** @type {?} */ onRenderingContext = onInit.pipe(switchMap(() => {
            // TODO this is a hack, but is there a better alternative?
            const /** @type {?} */ cmp = (/** @type {?} */ (vcRef.injector)).view.component;
            // access the rendering context property
            return /** @type {?} */ (cmp[KEY_ON_RENDERING_CONTEXT]);
        }));
        // availability of the base URL
        const /** @type {?} */ onDeliveryUrl = onRenderingContext.pipe(filter(Boolean), map(rc => rc.context.hub.deliveryUrl.origin), distinctUntilChanged());
        // compute the source set
        combineLatest(onDeliveryUrl, onImage).pipe(map(([origin, img]) => getSourceSet(img, origin)), distinctUntilChanged(), takeUntil(onDestroy))
            .subscribe(srcset => {
            // update the attribute
            const /** @type {?} */ el = parentRef.nativeElement;
            renderer.setAttribute(el, 'srcset', srcset);
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        super.ngOnInit();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        super.ngOnDestroy();
    }
}
SrcSetDirective.decorators = [
    { type: Directive, args: [{
                selector: '[wchSrcSet]'
            },] },
];
/** @nocollapse */
SrcSetDirective.ctorParameters = () => [
    { type: ViewContainerRef, },
    { type: ElementRef, },
    { type: Renderer2, },
];
SrcSetDirective.propDecorators = {
    "wchSrcSet": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class WchNgRenditionComponentsModule {
}
WchNgRenditionComponentsModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    RenditionDirective,
                    SrcSetDirective
                ],
                providers: [],
                exports: [
                    RenditionDirective,
                    SrcSetDirective
                ],
                entryComponents: []
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class WchNgRenditionServicesModule {
    /**
     * @param {?} parentModule
     */
    constructor(parentModule) {
    }
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: WchNgRenditionServicesModule,
            providers: []
        };
    }
}
WchNgRenditionServicesModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [],
                providers: [
                    WchRenditionService
                ],
                exports: [],
                entryComponents: []
            },] },
];
/** @nocollapse */
WchNgRenditionServicesModule.ctorParameters = () => [
    { type: WchNgRenditionServicesModule, decorators: [{ type: Optional }, { type: SkipSelf },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class WchNgRenditionModule {
    /**
     * @param {?} parentModule
     */
    constructor(parentModule) {
    }
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: WchNgRenditionModule,
            providers: []
        };
    }
}
WchNgRenditionModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    WchNgRenditionComponentsModule,
                    WchNgRenditionServicesModule
                ]
            },] },
];
/** @nocollapse */
WchNgRenditionModule.ctorParameters = () => [
    { type: WchNgRenditionModule, decorators: [{ type: Optional }, { type: SkipSelf },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const LOGGER$2 = 'WchIntersectionDirective';
const bHasIntersectionObserver = (typeof IntersectionObserver !== 'undefined');
const DEFAULT_ROOT_MARGIN = '0px';
const DEFAULT_THRESHOLD = 0;
class IntersectionDirective extends AbstractLifeCycleComponent {
    /**
     * @param {?} aElement
     * @param {?} aInjector
     */
    constructor(aElement, aInjector) {
        super();
        // capture the input field
        const /** @type {?} */ rootMarginSubject = new BehaviorSubject(DEFAULT_ROOT_MARGIN);
        const /** @type {?} */ thresholdSubject = new BehaviorSubject(DEFAULT_THRESHOLD);
        // attach
        const /** @type {?} */ setRootMargin = createSetterOnSubject(rootMarginSubject);
        const /** @type {?} */ setThreshold = createSetterOnSubject(thresholdSubject);
        // quick pointer
        const /** @type {?} */ that = this;
        const /** @type {?} */ wchLogger = aInjector.get(WchLoggerService);
        // logger
        const /** @type {?} */ logger = wchLogger.get(LOGGER$2);
        // test if we have a window
        if (bHasIntersectionObserver) {
            // get some dependencies
            const /** @type {?} */ zoneService = aInjector.get(ZoneService);
            // destroy
            const /** @type {?} */ onDestroy = that.onOnDestroy;
            // config
            const /** @type {?} */ onRootMargin = rootMarginSubject.pipe(filter(Boolean), distinctUntilChanged());
            const /** @type {?} */ onThreshold = thresholdSubject.pipe(distinctUntilChanged());
            // observer
            const /** @type {?} */ onOptions = combineLatest(onRootMargin, onThreshold).pipe(
            // construct the options object
            map(([rootMargin, threshold]) => ({ rootMargin, threshold })),
            // just log
            tap(opt => logger.info('Intersection options', opt)));
            // construct the observer
            const /** @type {?} */ onIntersectionObserverEntry = onOptions.pipe(
            // when the options change, reconnect
            switchMap(opt => Observable.create((ob) => {
                // construct the new observer
                const /** @type {?} */ intersectObserver = new IntersectionObserver(createConsumerOnSubject(ob), opt);
                // start consuming
                intersectObserver.observe(aElement.nativeElement);
                // shutdown
                return () => intersectObserver.disconnect();
            })),
            // make sure to subscribe in the angular zone
            zoneService.subscribeOutside(),
            // observe inside
            zoneService.observeInside(),
            // map the array into a sequence
            concatMap(identity),
            // log this
            tap(data => logger.info('intersection event', data)),
            // disconnect when the component gets destroyed
            takeUntil(onDestroy));
            // attach
            that.onIntersect = onIntersectionObserverEntry;
        }
        else {
            // warn about this
            logger.warn('IntersectionObserver not defined.');
            // nothing to do
            that.onIntersect = empty();
        }
        // define the setter
        Object.defineProperties(that, {
            'intersectRootMargin': setRootMargin,
            'intersectThreshold': setThreshold,
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        super.ngOnInit();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        super.ngOnDestroy();
    }
}
IntersectionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[wchIntersect]'
            },] },
    { type: Injectable },
];
/** @nocollapse */
IntersectionDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: Injector, decorators: [{ type: Inject, args: [Injector,] },] },
];
IntersectionDirective.propDecorators = {
    "wchIntersect": [{ type: Input },],
    "intersectRootMargin": [{ type: Input },],
    "intersectThreshold": [{ type: Input },],
    "onIntersect": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class WchNgIntersectionModule {
}
WchNgIntersectionModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    IntersectionDirective,
                ],
                exports: [
                    IntersectionDirective
                ]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { WchRenditionService, RenditionDirective, SrcSetDirective, WchNgRenditionModule, WchNgRenditionServicesModule, WchNgRenditionComponentsModule, WchNgIntersectionModule, IntersectionDirective };
//# sourceMappingURL=ibm-wch-sdk-ng-rendition.js.map
