import { __extends, __read } from 'tslib';
import { getRendition, createDimension, getSourceSet } from 'ibm-wch-sdk-rendition-utils';
import { WchLoggerService, AbstractLifeCycleComponent, ZoneService } from 'ibm-wch-sdk-ng';
export { AbstractLifeCycleComponent as ɵa } from 'ibm-wch-sdk-ng';
import { Inject, Injectable, Injector, Directive, ElementRef, EventEmitter, Input, Output, Renderer2, ViewContainerRef, NgModule, Optional, SkipSelf } from '@angular/core';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { combineLatest } from 'rxjs/observable/combineLatest';
import { fromEvent } from 'rxjs/observable/fromEvent';
import { of } from 'rxjs/observable/of';
import { debounceTime } from 'rxjs/operators/debounceTime';
import { distinctUntilChanged } from 'rxjs/operators/distinctUntilChanged';
import { filter } from 'rxjs/operators/filter';
import { map } from 'rxjs/operators/map';
import { mapTo } from 'rxjs/operators/mapTo';
import { shareReplay } from 'rxjs/operators/shareReplay';
import { startWith } from 'rxjs/operators/startWith';
import { switchMap } from 'rxjs/operators/switchMap';
import { switchMapTo } from 'rxjs/operators/switchMapTo';
import { takeUntil } from 'rxjs/operators/takeUntil';
import { tap } from 'rxjs/operators/tap';
import { ReplaySubject } from 'rxjs/ReplaySubject';
import { CommonModule } from '@angular/common';
import { createConsumerOnSubject, createSetterOnSubject, identity } from 'ibm-wch-sdk-utils';
import { Observable } from 'rxjs/Observable';
import { empty } from 'rxjs/observable/empty';
import { concatMap } from 'rxjs/operators/concatMap';

var LOGGER = 'WchRenditionService';
var WchRenditionService = /** @class */ (function () {
    function WchRenditionService(aInjector) {
        var that = this;
        var loggerService = aInjector.get(WchLoggerService);
        var logger = loggerService.get(LOGGER);
        function _getRenditionUrl(aWidth, aHeight, aImage) {
            return getRendition(aWidth, aHeight, aImage, logger).url;
        }
        function _getRendition(aWidth, aHeight, aImage) {
            return getRendition(aWidth, aHeight, aImage, logger);
        }
        that.getRenditionUrl = _getRenditionUrl;
        that.getRendition = _getRendition;
    }
    return WchRenditionService;
}());
WchRenditionService.decorators = [
    { type: Injectable },
];
WchRenditionService.ctorParameters = function () { return [
    { type: Injector, decorators: [{ type: Inject, args: [Injector,] },] },
]; };
var KEY_ON_RENDERING_CONTEXT = 'onRenderingContext';
var _mathRound = Math.round;
var jsonStringify = JSON.stringify;
function _equalDimension(aLeft, aRight) {
    return (aLeft === aRight) || ((aLeft.w === aRight.w) && (aLeft.h === aRight.h));
}
function _equalScaledImage(aLeft, aRight) {
    return (aLeft === aRight) || (_equalDimension(aLeft.img, aRight.img)
        && _equalDimension(aLeft.container, aRight.container) && (aLeft.url === aRight.url));
}
function _dimensionFromArray(aArray) {
    return createDimension(aArray[0], aArray[1]);
}
function _getMeasuredDimension(aElement, aRenderer, aLogger) {
    var ce = aElement;
    var w = 0, h = 0;
    while ((w <= 0) && ce) {
        ce = aRenderer.parentNode(ce);
        w = _mathRound(ce.clientWidth);
        h = _mathRound(ce.clientHeight);
    }
    aLogger.info('Measured dimension', w, 'x', h, 'via', ce);
    return createDimension(w, h);
}
var LOGGER$1 = 'WchRenditionDirective';
var RenditionDirective = /** @class */ (function (_super) {
    __extends(RenditionDirective, _super);
    function RenditionDirective(vcRef, parentRef, renderer, aInjector) {
        var _this = _super.call(this) || this;
        _this.onImage = new EventEmitter();
        var that = _this;
        var loggerService = aInjector.get(WchLoggerService);
        var logger = loggerService.get(LOGGER$1);
        logger.info('Constructing ...');
        var onDestroy = that.onOnDestroy.pipe(shareReplay(1));
        var onInit = that.onOnInit.pipe(shareReplay(1));
        var onImage = new ReplaySubject(1);
        var onRelWidth = new BehaviorSubject(0);
        var onRelHeight = new BehaviorSubject(0);
        var onImgWidth = new BehaviorSubject(0);
        var onImgHeight = new BehaviorSubject(0);
        Object.defineProperties(that, {
            'wchRendition': that.describeSetter(onImage),
            'relWidth': that.describeSetter(onRelWidth),
            'relHeight': that.describeSetter(onRelHeight),
            'imgWidth': that.describeSetter(onImgWidth),
            'imgHeight': that.describeSetter(onImgHeight)
        });
        var onCurrentElement = onInit.pipe(mapTo((parentRef.nativeElement)), filter(Boolean), distinctUntilChanged());
        var onConfiguredDimension = onInit.pipe(switchMapTo(combineLatest(onImgWidth, onImgHeight)), map(_dimensionFromArray), distinctUntilChanged(_equalDimension), tap(function (d) { return logger.info('config', d); }));
        var onRelDimension = onInit.pipe(switchMapTo(combineLatest(onRelWidth, onRelHeight)), map(_dimensionFromArray), distinctUntilChanged(_equalDimension), tap(function (d) { return logger.info('rel', d); }));
        var onMeasuredDimension = onCurrentElement.pipe(switchMap(function (ce) { return fromEvent(window, 'resize').pipe(startWith('start'), debounceTime(300), mapTo(_getMeasuredDimension(ce, renderer, logger))); }), distinctUntilChanged(_equalDimension), tap(function (d) { return logger.info('measured', d); }));
        var onSrcDimension = onConfiguredDimension.pipe(switchMap(function (d) {
            var dw = d.w, dh = d.h;
            return ((dw > 0) && (dh > 0)) ? of(d) : onMeasuredDimension.pipe(map(function (dm) {
                var mw = dm.w, mh = dm.h;
                return (dw > 0) ? createDimension(dw, mh) : ((dh > 0) ? createDimension(mw, dh) : dm);
            }));
        }), distinctUntilChanged(_equalDimension), tap(function (d) { return logger.info('src', d); }));
        var onDstDimension = onSrcDimension.pipe(switchMap(function (sd) {
            var sw = sd.w, sh = sd.h;
            return onRelDimension.pipe(map(function (rd) {
                var rw = rd.w, rh = rd.h;
                return (rw > 0) ? ((rh > 0) ? createDimension(rw * sh, rh * sw) : createDimension(rw * sh, sh))
                    : ((rh > 0) ? createDimension(sw, rh * sw) : sd);
            }));
        }), distinctUntilChanged(_equalDimension), tap(function (d) { return logger.info('dst', d); }));
        var onScaledImage = combineLatest(onImage.pipe(filter(Boolean)), onDstDimension).pipe(map(function (_a) {
            var _b = __read(_a, 2), img = _b[0], d = _b[1];
            return getRendition(d.w, d.h, img, logger);
        }), distinctUntilChanged(_equalScaledImage), tap(function (d) { return logger.info('url', jsonStringify(d)); }));
        var onRenderingContext = onInit.pipe(switchMap(function () {
            var cmp = ((vcRef.injector)).view.component;
            return (cmp[KEY_ON_RENDERING_CONTEXT]);
        }));
        var onDeliveryUrl = onRenderingContext.pipe(filter(Boolean), map(function (rc) { return rc.context.hub.deliveryUrl.origin; }), distinctUntilChanged());
        combineLatest(onDeliveryUrl, onScaledImage).pipe(map(function (_a) {
            var _b = __read(_a, 2), base = _b[0], img = _b[1];
            return Object.assign({}, img, { url: base + img.url });
        }), distinctUntilChanged(_equalScaledImage), tap(function (img) { return logger.info('result', img); }), takeUntil(onDestroy))
            .subscribe(function (img) {
            var el = parentRef.nativeElement;
            renderer.setAttribute(el, 'src', img.url);
            renderer.setAttribute(el, 'width', img.container.w.toString());
            renderer.setAttribute(el, 'height', img.container.h.toString());
            that.onImage.emit(img);
        });
        return _this;
    }
    RenditionDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
    };
    RenditionDirective.prototype.ngOnDestroy = function () {
        _super.prototype.ngOnDestroy.call(this);
    };
    return RenditionDirective;
}(AbstractLifeCycleComponent));
RenditionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[wchRendition]'
            },] },
];
RenditionDirective.ctorParameters = function () { return [
    { type: ViewContainerRef, },
    { type: ElementRef, },
    { type: Renderer2, },
    { type: Injector, decorators: [{ type: Inject, args: [Injector,] },] },
]; };
RenditionDirective.propDecorators = {
    "wchRendition": [{ type: Input },],
    "relHeight": [{ type: Input },],
    "relWidth": [{ type: Input },],
    "imgWidth": [{ type: Input },],
    "imgHeight": [{ type: Input },],
    "onImage": [{ type: Output },],
};
var SrcSetDirective = /** @class */ (function (_super) {
    __extends(SrcSetDirective, _super);
    function SrcSetDirective(vcRef, parentRef, renderer) {
        var _this = _super.call(this) || this;
        var that = _this;
        var onDestroy = that.onOnDestroy;
        var onInit = that.onOnInit;
        var onImage = new ReplaySubject(1);
        Object.defineProperties(that, {
            'wchSrcSet': _this.describeSetter(onImage)
        });
        var onRenderingContext = onInit.pipe(switchMap(function () {
            var cmp = ((vcRef.injector)).view.component;
            return (cmp[KEY_ON_RENDERING_CONTEXT]);
        }));
        var onDeliveryUrl = onRenderingContext.pipe(filter(Boolean), map(function (rc) { return rc.context.hub.deliveryUrl.origin; }), distinctUntilChanged());
        combineLatest(onDeliveryUrl, onImage).pipe(map(function (_a) {
            var _b = __read(_a, 2), origin = _b[0], img = _b[1];
            return getSourceSet(img, origin);
        }), distinctUntilChanged(), takeUntil(onDestroy))
            .subscribe(function (srcset) {
            var el = parentRef.nativeElement;
            renderer.setAttribute(el, 'srcset', srcset);
        });
        return _this;
    }
    SrcSetDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
    };
    SrcSetDirective.prototype.ngOnDestroy = function () {
        _super.prototype.ngOnDestroy.call(this);
    };
    return SrcSetDirective;
}(AbstractLifeCycleComponent));
SrcSetDirective.decorators = [
    { type: Directive, args: [{
                selector: '[wchSrcSet]'
            },] },
];
SrcSetDirective.ctorParameters = function () { return [
    { type: ViewContainerRef, },
    { type: ElementRef, },
    { type: Renderer2, },
]; };
SrcSetDirective.propDecorators = {
    "wchSrcSet": [{ type: Input },],
};
var WchNgRenditionComponentsModule = /** @class */ (function () {
    function WchNgRenditionComponentsModule() {
    }
    return WchNgRenditionComponentsModule;
}());
WchNgRenditionComponentsModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    RenditionDirective,
                    SrcSetDirective
                ],
                providers: [],
                exports: [
                    RenditionDirective,
                    SrcSetDirective
                ],
                entryComponents: []
            },] },
];
var WchNgRenditionServicesModule = /** @class */ (function () {
    function WchNgRenditionServicesModule(parentModule) {
    }
    WchNgRenditionServicesModule.forRoot = function () {
        return {
            ngModule: WchNgRenditionServicesModule,
            providers: []
        };
    };
    return WchNgRenditionServicesModule;
}());
WchNgRenditionServicesModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [],
                providers: [
                    WchRenditionService
                ],
                exports: [],
                entryComponents: []
            },] },
];
WchNgRenditionServicesModule.ctorParameters = function () { return [
    { type: WchNgRenditionServicesModule, decorators: [{ type: Optional }, { type: SkipSelf },] },
]; };
var WchNgRenditionModule = /** @class */ (function () {
    function WchNgRenditionModule(parentModule) {
    }
    WchNgRenditionModule.forRoot = function () {
        return {
            ngModule: WchNgRenditionModule,
            providers: []
        };
    };
    return WchNgRenditionModule;
}());
WchNgRenditionModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    WchNgRenditionComponentsModule,
                    WchNgRenditionServicesModule
                ]
            },] },
];
WchNgRenditionModule.ctorParameters = function () { return [
    { type: WchNgRenditionModule, decorators: [{ type: Optional }, { type: SkipSelf },] },
]; };
var LOGGER$2 = 'WchIntersectionDirective';
var bHasIntersectionObserver = (typeof IntersectionObserver !== 'undefined');
var DEFAULT_ROOT_MARGIN = '0px';
var DEFAULT_THRESHOLD = 0;
var IntersectionDirective = /** @class */ (function (_super) {
    __extends(IntersectionDirective, _super);
    function IntersectionDirective(aElement, aInjector) {
        var _this = _super.call(this) || this;
        var rootMarginSubject = new BehaviorSubject(DEFAULT_ROOT_MARGIN);
        var thresholdSubject = new BehaviorSubject(DEFAULT_THRESHOLD);
        var setRootMargin = createSetterOnSubject(rootMarginSubject);
        var setThreshold = createSetterOnSubject(thresholdSubject);
        var that = _this;
        var wchLogger = aInjector.get(WchLoggerService);
        var logger = wchLogger.get(LOGGER$2);
        if (bHasIntersectionObserver) {
            var zoneService = aInjector.get(ZoneService);
            var onDestroy = that.onOnDestroy;
            var onRootMargin = rootMarginSubject.pipe(filter(Boolean), distinctUntilChanged());
            var onThreshold = thresholdSubject.pipe(distinctUntilChanged());
            var onOptions = combineLatest(onRootMargin, onThreshold).pipe(map(function (_a) {
                var _b = __read(_a, 2), rootMargin = _b[0], threshold = _b[1];
                return ({ rootMargin: rootMargin, threshold: threshold });
            }), tap(function (opt) { return logger.info('Intersection options', opt); }));
            var onIntersectionObserverEntry = onOptions.pipe(switchMap(function (opt) { return Observable.create(function (ob) {
                var intersectObserver = new IntersectionObserver(createConsumerOnSubject(ob), opt);
                intersectObserver.observe(aElement.nativeElement);
                return function () { return intersectObserver.disconnect(); };
            }); }), zoneService.subscribeOutside(), zoneService.observeInside(), concatMap(identity), tap(function (data) { return logger.info('intersection event', data); }), takeUntil(onDestroy));
            that.onIntersect = onIntersectionObserverEntry;
        }
        else {
            logger.warn('IntersectionObserver not defined.');
            that.onIntersect = empty();
        }
        Object.defineProperties(that, {
            'intersectRootMargin': setRootMargin,
            'intersectThreshold': setThreshold,
        });
        return _this;
    }
    IntersectionDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
    };
    IntersectionDirective.prototype.ngOnDestroy = function () {
        _super.prototype.ngOnDestroy.call(this);
    };
    return IntersectionDirective;
}(AbstractLifeCycleComponent));
IntersectionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[wchIntersect]'
            },] },
    { type: Injectable },
];
IntersectionDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: Injector, decorators: [{ type: Inject, args: [Injector,] },] },
]; };
IntersectionDirective.propDecorators = {
    "wchIntersect": [{ type: Input },],
    "intersectRootMargin": [{ type: Input },],
    "intersectThreshold": [{ type: Input },],
    "onIntersect": [{ type: Output },],
};
var WchNgIntersectionModule = /** @class */ (function () {
    function WchNgIntersectionModule() {
    }
    return WchNgIntersectionModule;
}());
WchNgIntersectionModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    IntersectionDirective,
                ],
                exports: [
                    IntersectionDirective
                ]
            },] },
];

export { WchRenditionService, RenditionDirective, SrcSetDirective, WchNgRenditionModule, WchNgRenditionServicesModule, WchNgRenditionComponentsModule, WchNgIntersectionModule, IntersectionDirective };
//# sourceMappingURL=ibm-wch-sdk-ng-rendition.js.map
